# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'DownloadStatistic'
        db.create_table(u'download_stats_downloadstatistic', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('download_url', self.gf('django.db.models.fields.CharField')(max_length=512)),
            ('count', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal(u'download_stats', ['DownloadStatistic'])


    def backwards(self, orm):
        # Deleting model 'DownloadStatistic'
        db.delete_table(u'download_stats_downloadstatistic')


    models = {
        u'download_stats.downloadstatistic': {
            'Meta': {'object_name': 'DownloadStatistic'},
            'count': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'download_url': ('django.db.models.fields.CharField', [], {'max_length': '512'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['download_stats']



"""URLs for the ``download_stats`` app."""
from django.conf.urls.defaults import patterns, url

from .settings import DOWNLOAD_URL
from .views import DownloadView


urlpatterns = patterns(
    '',
    url(r'{}/(?P<requested_file>.+)'.format(DOWNLOAD_URL.replace('/', '')),
        DownloadView.as_view(),
        name='download_view'),
)



"""Factories for the ``download_stats`` app."""
import factory

from ..models import DownloadStatistic


class DownloadStatisticFactory(factory.Factory):
    """Factory for the ``DownloadStatistic`` model."""
    FACTORY_FOR = DownloadStatistic

    download_url = factory.Sequence(lambda n: 'url-{}'.format(n))
    count = 10



"""URLs to run the tests."""
from django.conf import settings
from django.conf.urls.defaults import patterns, include, url
from django.contrib import admin
from django.views.generic import TemplateView


admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
    url('^$',
        TemplateView.as_view(template_name='dummy_home.html'),
        name='dummy_home'),
    url(r'^', include('download_stats.urls')),
)

if settings.DEBUG:
    urlpatterns += patterns(
        '',
        (r'%s(?P<path>.*)' % settings.MEDIA_URL[1:], 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),  # NOQA
    )



"""Models of the ``download_stats`` app."""
from django.db import models
from django.utils.translation import ugettext_lazy as _


class DownloadStatistic(models.Model):
    """
    Holds the information about how often a certain file was downloaded.

    :download_url: The URL from where the file was downloaded.
    :count: The amount of times this URL was clicked.

    """

    download_url = models.CharField(
        verbose_name=_('Download URL'),
        max_length=512,
    )

    count = models.PositiveIntegerField(
        verbose_name=_('Count'),
    )

    def __unicode__(self):
        return '{} ({})'.format(
            self.download_url[:50] + '...' if len(self.download_url) > 50
            else self.download_url, self.count)



"""Views for the ``download_stats`` app."""
import os

from django.conf import settings
from django.db.models import F
from django.http import Http404, HttpResponse
from django.views.generic import View
from django.utils.encoding import smart_str

from .models import DownloadStatistic


class DownloadView(View):
    """View that increments download counts and serves the files."""
    def dispatch(self, request, *args, **kwargs):
        self.requested_file = kwargs.get('requested_file')
        self.file_name = os.path.basename(self.requested_file)
        self.full_file_path = os.path.join(settings.MEDIA_ROOT,
                                           self.requested_file)
        if not self.requested_file or not os.path.exists(self.full_file_path):
            raise Http404

        return super(DownloadView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        with open(self.full_file_path) as f:
            response = HttpResponse(content=f.read(),
                                    content_type='application/force-download')
        response['Content-Disposition'] = 'attachment; filename={}'.format(
            smart_str(self.file_name))
        # the usual case is that there is already at least one download of a
        # file, so only on creation we would trigger an additional query
        if not DownloadStatistic.objects.filter(
                download_url=self.full_file_path).update(count=F('count')+1):
            DownloadStatistic.objects.create(download_url=self.full_file_path,
                                             count=1)
        return response



"""Context processors of the ``download_stats`` app."""
from .settings import DOWNLOAD_URL


def download_url(request):
    """Adds the DOWNLOAD_URL setting to the context."""
    return {'DOWNLOAD_URL': DOWNLOAD_URL}



"""Admin classes for the ``download_stats`` app."""
from django.contrib import admin

from .models import DownloadStatistic


class DownloadStatisticAdmin(admin.ModelAdmin):
    """Custom admin for the ``DownloadStatistic`` model."""
    list_display = ('download_url', 'count')
    search_fields = ['download_url', ]


admin.site.register(DownloadStatistic, DownloadStatisticAdmin)



"""Tests for the context processors of the ``download_stats`` app."""
from django.test import TestCase

from ..context_processors import download_url


class DownloadURLTestCase(TestCase):
    """Tests for the ``download_url`` context processor."""
    longMessage = True

    def test_download_url(self):
        self.assertEqual(download_url({}), {'DOWNLOAD_URL': '/downloads/'})



"""Tests for the models of the ``download_stats`` app."""
from django.test import TestCase

from .factories import DownloadStatisticFactory


class DownloadStatisticTestCase(TestCase):
    """Tests for the ``DownloadStatistic`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test instantiation of the ``DownloadStatistic`` model."""
        downloadstatistic = DownloadStatisticFactory()
        self.assertTrue(downloadstatistic.pk)



"""Settings that need to be set in order to run the tests."""
import os

DEBUG = True

SITE_ID = 1

APP_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '..'))


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

ROOT_URLCONF = 'download_stats.tests.urls'

STATIC_URL = '/static/'
MEDIA_URL = '/media/'
STATIC_ROOT = os.path.join(APP_ROOT, '../static')
MEDIA_ROOT = os.path.join(APP_ROOT, '../media')
STATICFILES_DIRS = (
    os.path.join(APP_ROOT, 'static'),
)

TEMPLATE_DIRS = (
    os.path.join(APP_ROOT, 'templates'),
)

TEMPLATE_CONTEXT_PROCESSORS = [
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.core.context_processors.tz",
    "django.core.context_processors.request",
    "django.contrib.messages.context_processors.messages",
    'download_stats.context_processors.download_url',
]

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    os.path.join(APP_ROOT, 'tests/coverage'))
COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]

EXTERNAL_APPS = [
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'django_nose',
]

INTERNAL_APPS = [
    'download_stats',
]

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS

COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS

SECRET_KEY = 'foobar'



"""Tests for the views of the ``download_stats`` app."""
import os
import shutil

from django.conf import settings
from django.test import TestCase

from django_libs.tests.mixins import ViewTestMixin

from download_stats.models import DownloadStatistic


class DownloadViewTestCase(ViewTestMixin, TestCase):
    """Tests for the ``DownloadView`` view class."""
    longMessage = True

    def get_view_name(self):
        return 'download_view'

    def get_view_kwargs(self):
        return self.view_kwargs  # defined in setUp method

    def setUp(self):
        # set up media folder and download temp folder. The test requirements
        # should serve as test file
        self.temp_folder = os.path.join(settings.MEDIA_ROOT, 'tmp')
        self.req_file = os.path.join(settings.APP_ROOT,
                                     '../test_requirements.txt')
        self.temp_file = os.path.join(settings.MEDIA_ROOT,
                                      'test_requirements.txt')
        try:
            os.makedirs(self.temp_folder)
        except OSError:
            pass
        shutil.copy(self.req_file, self.temp_folder)

        self.view_kwargs = {'requested_file': 'test_requirements.txt'}

    def tearDown(self):
        shutil.rmtree(self.temp_folder)

    def test_view(self):
        self.is_callable()
        self.assertEqual(DownloadStatistic.objects.count(), 1, msg=(
            'The view hasn\'t created the expected download statisics.'))
        stat = DownloadStatistic.objects.get()
        self.assertEqual(stat.count, 1, msg=(
            'The view set the wrong amount of downloads'))

        self.is_callable()
        self.assertEqual(DownloadStatistic.objects.count(), 1, msg=(
            'The view did not get the persistant stat properly.'))
        stat = DownloadStatistic.objects.get()
        self.assertEqual(stat.count, 2, msg=(
            'The view set the wrong amount of downloads'))

        self.view_kwargs.update({'requested_file': 'not_existing_file.xyz'})
        self.is_not_callable()

        self.view_kwargs = {'requested_file': '../../no_valid.file'}
        self.is_not_callable(message='This file should not be accessable.')
